@@ -1,7 +1,7 @@
 /*
  * augeas.c: the core data structure for storing key/value pairs
  *
- * Copyright (C) 2007-2010 David Lutterkort
+ * Copyright (C) 2007-2011 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -33,6 +33,7 @@
 #include <string.h>
 #include <stdarg.h>
 #include <locale.h>
+#include <libxml/tree.h>
 
 /* Some popular labels that we use in /augeas */
 static const char *const s_augeas = "augeas";
@@ -48,6 +49,8 @@ static const char *const s_vars   = "variables";
 #define AUGEAS_META_PATHX_FUNC AUGEAS_META_TREE "/version/pathx/functions"
 
 static const char *const static_nodes[][2] = {
+    { AUGEAS_FILES_TREE, NULL },
+    { AUGEAS_META_TREE "/variables", NULL },
     { AUGEAS_META_TREE "/version", PACKAGE_VERSION },
     { AUGEAS_META_TREE "/version/save/mode[1]", AUG_SAVE_BACKUP_TEXT },
     { AUGEAS_META_TREE "/version/save/mode[2]", AUG_SAVE_NEWFILE_TEXT },
@@ -74,7 +77,8 @@ static const char *const errcodes[] = {
     "Multiple transforms",                              /* AUG_EMXFM */
     "Node has no span info",                            /* AUG_ENOSPAN */
     "Cannot move node into its descendant",             /* AUG_EMVDESC */
-    "Failed to execute command"                         /* AUG_ECMDRUN */
+    "Failed to execute command",                        /* AUG_ECMDRUN */
+    "Invalid argument in function call"                 /* AUG_EBADARG */
 };
 
 static void tree_mark_dirty(struct tree *tree) {
@@ -137,7 +141,7 @@ struct tree *tree_find(struct augeas *aug, const char *path) {
     struct tree *result = NULL;
     int r;
 
-    p = pathx_aug_parse(aug, aug->origin, path, true);
+    p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), path, true);
     ERR_BAIL(aug);
 
     r = pathx_find_one(p, &result);
@@ -157,7 +161,7 @@ struct tree *tree_find_cr(struct augeas *aug, const char *path) {
     struct tree *result = NULL;
     int r;
 
-    p = pathx_aug_parse(aug, aug->origin, path, true);
+    p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), path, true);
     ERR_BAIL(aug);
 
     r = pathx_expand_tree(p, &result);
@@ -235,6 +239,7 @@ static void store_pathx_error(const struct augeas *aug) {
 
 struct pathx *pathx_aug_parse(const struct augeas *aug,
                               struct tree *tree,
+                              struct tree *root_ctx,
                               const char *path, bool need_nodeset) {
     struct pathx *result;
     struct error *err = err_of_aug(aug);
@@ -242,28 +247,46 @@ struct pathx *pathx_aug_parse(const struct augeas *aug,
     if (tree == NULL)
         tree = aug->origin;
 
-    pathx_parse(tree, err, path, need_nodeset, aug->symtab, &result);
+    pathx_parse(tree, err, path, need_nodeset, aug->symtab, root_ctx, &result);
     return result;
 }
 
-static const char *init_root(const char *root0) {
-    char *root;
+/* Find the tree stored in AUGEAS_CONTEXT */
+struct tree *tree_root_ctx(const struct augeas *aug) {
+    struct pathx *p = NULL;
+    struct tree *match = NULL;
+    const char *ctx_path;
+    int r;
 
-    if (root0 == NULL)
-        root0 = getenv(AUGEAS_ROOT_ENV);
-    if (root0 == NULL || root0[0] == '\0')
-        root0 = "/";
-    root = strdup(root0);
-    if (root == NULL)
-        return NULL;
-    if (root[strlen(root)-1] != SEP) {
-        if (REALLOC_N(root, strlen(root) + 2) == -1) {
-            FREE(root);
-            return NULL;
-        }
-        strcat(root, "/");
-    }
-    return root;
+    p = pathx_aug_parse(aug, aug->origin, NULL, AUGEAS_CONTEXT, true);
+    ERR_BAIL(aug);
+
+    r = pathx_find_one(p, &match);
+    ERR_THROW(r > 1, aug, AUG_EMMATCH,
+              "There are %d nodes matching %s, expecting one",
+              r, AUGEAS_CONTEXT);
+
+    if (match == NULL || match->value == NULL || *match->value == '\0')
+        goto error;
+
+    /* Clean via augrun's helper to ensure it's valid */
+    ctx_path = cleanpath(match->value);
+    free_pathx(p);
+
+    p = pathx_aug_parse(aug, aug->origin, NULL, ctx_path, true);
+    ERR_BAIL(aug);
+
+    r = pathx_find_one(p, &match);
+    ERR_THROW(r > 1, aug, AUG_EMMATCH,
+              "There are %d nodes matching the context %s, expecting one",
+              r, ctx_path);
+
+ done:
+    free_pathx(p);
+    return match;
+ error:
+    match = NULL;
+    goto done;
 }
 
 struct tree *tree_append(struct tree *parent,
@@ -274,42 +297,55 @@ struct tree *tree_append(struct tree *parent,
     return result;
 }
 
+static struct tree *tree_append_s(struct tree *parent,
+                                  const char *l0, char *v) {
+    struct tree *result;
+    char *l = strdup(l0);
+
+    if (l == NULL)
+        return NULL;
+    result = tree_append(parent, l, v);
+    if (result == NULL)
+        free(l);
+    return result;
+}
+
 static struct tree *tree_from_transform(struct augeas *aug,
                                         const char *modname,
                                         struct transform *xfm) {
     struct tree *meta = tree_child_cr(aug->origin, s_augeas);
-    struct tree *load = NULL, *txfm = NULL;
-    char *m = NULL, *q = NULL;
+    struct tree *load = NULL, *txfm = NULL, *t;
+    char *v = NULL;
+    int r;
 
-    if (meta == NULL)
-        goto error;
+    ERR_NOMEM(meta == NULL, aug);
 
     load = tree_child_cr(meta, s_load);
-    if (load == NULL)
-            goto error;
-    if (modname != NULL) {
-        m = strdup(modname);
-        if (m == NULL)
-            goto error;
-    } else {
-        m = strdup("_");
-    }
-    txfm = tree_append(load, m, NULL);
-    if (txfm == NULL)
-        goto error;
+    ERR_NOMEM(load == NULL, aug);
 
-    if (asprintf(&q, "@%s", modname) < 0)
-        goto error;
+    if (modname == NULL)
+        modname = "_";
+
+    txfm = tree_append_s(load, modname, NULL);
+    ERR_NOMEM(txfm == NULL, aug);
+
+    r = asprintf(&v, "@%s", modname);
+    ERR_NOMEM(r < 0, aug);
+
+    t = tree_append_s(txfm, "lens", v);
+    ERR_NOMEM(t == NULL, aug);
+    v = NULL;
 
-    m = strdup("lens");
-    tree_append(txfm, m, q);
     list_for_each(f, xfm->filter) {
-        char *glob = strdup(f->glob->str);
-        char *l = f->include ? strdup("incl") : strdup("excl");
-        tree_append(txfm, l, glob);
+        const char *l = f->include ? strdup("incl") : strdup("excl");
+        v = strdup(f->glob->str);
+        ERR_NOMEM(v == NULL, aug);
+        t = tree_append_s(txfm, l, v);
+        ERR_NOMEM(t == NULL, aug);
     }
     return txfm;
  error:
+    free(v);
     tree_unlink(txfm);
     return NULL;
 }
@@ -366,11 +402,102 @@ void api_exit(const struct augeas *aug) {
     }
 }
 
+static int init_root(struct augeas *aug, const char *root0) {
+    if (root0 == NULL)
+        root0 = getenv(AUGEAS_ROOT_ENV);
+    if (root0 == NULL || root0[0] == '\0')
+        root0 = "/";
+
+    aug->root = strdup(root0);
+    if (aug->root == NULL)
+        return -1;
+
+    if (aug->root[strlen(aug->root)-1] != SEP) {
+        if (REALLOC_N(aug->root, strlen(aug->root) + 2) < 0)
+            return -1;
+        strcat((char *) aug->root, "/");
+    }
+    return 0;
+}
+
+static int init_loadpath(struct augeas *aug, const char *loadpath) {
+    int r;
+
+    aug->modpathz = NULL;
+    aug->nmodpath = 0;
+    if (loadpath != NULL) {
+        r = argz_add_sep(&aug->modpathz, &aug->nmodpath,
+                         loadpath, PATH_SEP_CHAR);
+        if (r != 0)
+            return -1;
+    }
+    char *env = getenv(AUGEAS_LENS_ENV);
+    if (env != NULL) {
+        r = argz_add_sep(&aug->modpathz, &aug->nmodpath,
+                         env, PATH_SEP_CHAR);
+        if (r != 0)
+            return -1;
+    }
+    if (!(aug->flags & AUG_NO_STDINC)) {
+        r = argz_add(&aug->modpathz, &aug->nmodpath, AUGEAS_LENS_DIR);
+        if (r != 0)
+            return -1;
+        r = argz_add(&aug->modpathz, &aug->nmodpath,
+                     AUGEAS_LENS_DIST_DIR);
+        if (r != 0)
+            return -1;
+    }
+    /* Clean up trailing slashes */
+    if (aug->nmodpath > 0) {
+        argz_stringify(aug->modpathz, aug->nmodpath, PATH_SEP_CHAR);
+        char *s, *t;
+        const char *e = aug->modpathz + strlen(aug->modpathz);
+        for (s = aug->modpathz, t = aug->modpathz; s < e; s++) {
+            char *p = s;
+            if (*p == '/') {
+                while (*p == '/') p += 1;
+                if (*p == '\0' || *p == PATH_SEP_CHAR)
+                    s = p;
+            }
+            if (t != s)
+                *t++ = *s;
+            else
+                t += 1;
+        }
+        if (t != s) {
+            *t = '\0';
+        }
+        s = aug->modpathz;
+        aug->modpathz = NULL;
+        r = argz_create_sep(s, PATH_SEP_CHAR, &aug->modpathz,
+                            &aug->nmodpath);
+        free(s);
+        if (r != 0)
+            return -1;
+    }
+    return 0;
+}
+
+static void init_save_mode(struct augeas *aug) {
+    const char *v = AUG_SAVE_OVERWRITE_TEXT;
+
+    if (aug->flags & AUG_SAVE_NEWFILE) {
+        v = AUG_SAVE_NEWFILE_TEXT;
+    } else if (aug->flags & AUG_SAVE_BACKUP) {
+        v = AUG_SAVE_BACKUP_TEXT;
+    } else if (aug->flags & AUG_SAVE_NOOP) {
+        v = AUG_SAVE_NOOP_TEXT;
+    }
+
+    aug_set(aug, AUGEAS_META_SAVE_MODE, v);
+}
+
 struct augeas *aug_init(const char *root, const char *loadpath,
                         unsigned int flags) {
     struct augeas *result;
     struct tree *tree_root = make_tree(NULL, NULL, NULL, NULL);
     int r;
+    bool close_on_error = true;
 
     if (tree_root == NULL)
         return NULL;
@@ -397,84 +524,39 @@ struct augeas *aug_init(const char *root, const char *loadpath,
 
     result->flags = flags;
 
-    result->root = init_root(root);
+    r = init_root(result, root);
+    ERR_NOMEM(r < 0, result);
 
     result->origin->children->label = strdup(s_augeas);
 
-    result->modpathz = NULL;
-    result->nmodpath = 0;
-    if (loadpath != NULL) {
-        r = argz_add_sep(&result->modpathz, &result->nmodpath,
-                         loadpath, PATH_SEP_CHAR);
-        ERR_NOMEM(r != 0, result);
-    }
-    char *env = getenv(AUGEAS_LENS_ENV);
-    if (env != NULL) {
-        r = argz_add_sep(&result->modpathz, &result->nmodpath,
-                         env, PATH_SEP_CHAR);
-        ERR_NOMEM(r != 0, result);
-    }
-    if (!(flags & AUG_NO_STDINC)) {
-        r = argz_add(&result->modpathz, &result->nmodpath, AUGEAS_LENS_DIR);
-        ERR_NOMEM(r != 0, result);
-        r = argz_add(&result->modpathz, &result->nmodpath,
-                     AUGEAS_LENS_DIST_DIR);
-        ERR_NOMEM(r != 0, result);
-    }
-    /* Clean up trailing slashes */
-    if (result->nmodpath > 0) {
-        argz_stringify(result->modpathz, result->nmodpath, PATH_SEP_CHAR);
-        char *s, *t;
-        const char *e = result->modpathz + strlen(result->modpathz);
-        for (s = result->modpathz, t = result->modpathz; s < e; s++) {
-            char *p = s;
-            if (*p == '/') {
-                while (*p == '/') p += 1;
-                if (*p == '\0' || *p == PATH_SEP_CHAR)
-                    s = p;
-            }
-            if (t != s)
-                *t++ = *s;
-            else
-                t += 1;
-        }
-        if (t != s) {
-            *t = '\0';
-        }
-        s = result->modpathz;
-        r = argz_create_sep(s, PATH_SEP_CHAR, &result->modpathz,
-                            &result->nmodpath);
-        free(s);
-        ERR_NOMEM(r != 0, result);
-    }
+    /* We are now initialized enough that we can dare return RESULT even
+     * when we encounter errors if the caller so wishes */
+    close_on_error = !(flags & AUG_NO_ERR_CLOSE);
+
+    r = init_loadpath(result, loadpath);
+    ERR_NOMEM(r < 0, result);
 
     /* We report the root dir in AUGEAS_META_ROOT, but we only use the
        value we store internally, to avoid any problems with
        AUGEAS_META_ROOT getting changed. */
     aug_set(result, AUGEAS_META_ROOT, result->root);
+    ERR_BAIL(result);
 
-    for (int i=0; i < ARRAY_CARDINALITY(static_nodes); i++)
+    /* Set the default path context */
+    aug_set(result, AUGEAS_CONTEXT, AUG_CONTEXT_DEFAULT);
+    ERR_BAIL(result);
+
+    for (int i=0; i < ARRAY_CARDINALITY(static_nodes); i++) {
         aug_set(result, static_nodes[i][0], static_nodes[i][1]);
-
-    if (flags & AUG_SAVE_NEWFILE) {
-        aug_set(result, AUGEAS_META_SAVE_MODE, AUG_SAVE_NEWFILE_TEXT);
-    } else if (flags & AUG_SAVE_BACKUP) {
-        aug_set(result, AUGEAS_META_SAVE_MODE, AUG_SAVE_BACKUP_TEXT);
-    } else if (flags & AUG_SAVE_NOOP) {
-        aug_set(result, AUGEAS_META_SAVE_MODE, AUG_SAVE_NOOP_TEXT);
-    } else {
-        aug_set(result, AUGEAS_META_SAVE_MODE, AUG_SAVE_OVERWRITE_TEXT);
+        ERR_BAIL(result);
     }
 
-    if (flags & AUG_ENABLE_SPAN) {
-        aug_set(result, AUGEAS_SPAN_OPTION, AUG_ENABLE);
-    } else {
-        aug_set(result, AUGEAS_SPAN_OPTION, AUG_DISABLE);
-    }
+    init_save_mode(result);
+    ERR_BAIL(result);
 
-    /* Make sure we always have /files and /augeas/variables */
-    tree_path_cr(result->origin, 1, s_files);
-    tree_path_cr(result->origin, 2, s_augeas, s_vars);
+    const char *v = (flags & AUG_ENABLE_SPAN) ? AUG_ENABLE : AUG_DISABLE;
+    aug_set(result, AUGEAS_SPAN_OPTION, v);
+    ERR_BAIL(result);
 
     if (interpreter_init(result) == -1)
         goto error;
@@ -484,6 +566,7 @@ struct augeas *aug_init(const char *root, const char *loadpath,
         if (xform == NULL)
             continue;
         tree_from_transform(result, modl->name, xform);
+        ERR_BAIL(result);
     }
     if (!(result->flags & AUG_NO_LOAD))
         if (aug_load(result) < 0)
@@ -493,9 +576,13 @@ struct augeas *aug_init(const char *root, const char *loadpath,
     return result;
 
  error:
-    api_exit(result);
-    aug_close(result);
-    return NULL;
+    if (close_on_error) {
+        aug_close(result);
+        result = NULL;
+    }
+    if (result != NULL && result->api_entries > 0)
+        api_exit(result);
+    return result;
 }
 
 void tree_unlink_children(struct augeas *aug, struct tree *tree) {
@@ -643,7 +730,7 @@ int aug_get(const struct augeas *aug, const char *path, const char **value) {
 
     api_entry(aug);
 
-    p = pathx_aug_parse(aug, aug->origin, path, true);
+    p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), path, true);
     ERR_BAIL(aug);
 
     if (value != NULL)
@@ -693,7 +780,7 @@ int aug_defvar(augeas *aug, const char *name, const char *expr) {
     if (expr == NULL) {
         result = pathx_symtab_undefine(&(aug->symtab), name);
     } else {
-        p = pathx_aug_parse(aug, aug->origin, expr, false);
+        p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), expr, false);
         ERR_BAIL(aug);
         result = pathx_symtab_define(&(aug->symtab), name, p);
     }
@@ -721,7 +808,7 @@ int aug_defnode(augeas *aug, const char *name, const char *expr,
     if (created == NULL)
         created = &cr;
 
-    p = pathx_aug_parse(aug, aug->origin, expr, false);
+    p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), expr, false);
     ERR_BAIL(aug);
 
     if (pathx_first(p) == NULL) {
@@ -778,7 +865,12 @@ int aug_set(struct augeas *aug, const char *path, const char *value) {
 
     api_entry(aug);
 
-    p = pathx_aug_parse(aug, aug->origin, path, true);
+    /* Get-out clause, in case context is broken */
+    struct tree *root_ctx = NULL;
+    if (STRNEQ(path, AUGEAS_CONTEXT))
+        root_ctx = tree_root_ctx(aug);
+
+    p = pathx_aug_parse(aug, aug->origin, root_ctx, path, true);
     ERR_BAIL(aug);
 
     result = tree_set(p, value) == NULL ? -1 : 0;
@@ -799,7 +891,7 @@ int aug_setm(struct augeas *aug, const char *base,
 
     api_entry(aug);
 
-    bx = pathx_aug_parse(aug, aug->origin, base, true);
+    bx = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), base, true);
     ERR_BAIL(aug);
 
     if (sub != NULL && STREQ(sub, "."))
@@ -809,7 +901,7 @@ int aug_setm(struct augeas *aug, const char *base,
     for (bt = pathx_first(bx); bt != NULL; bt = pathx_next(bx)) {
         if (sub != NULL) {
             /* Handle subnodes of BT */
-            sx = pathx_aug_parse(aug, bt, sub, true);
+            sx = pathx_aug_parse(aug, bt, NULL, sub, true);
             ERR_BAIL(aug);
             if (pathx_first(sx) != NULL) {
                 /* Change existing subnodes matching SUB */
@@ -877,7 +969,7 @@ int aug_insert(struct augeas *aug, const char *path, const char *label,
 
     api_entry(aug);
 
-    p = pathx_aug_parse(aug, aug->origin, path, true);
+    p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), path, true);
     ERR_BAIL(aug);
 
     result = tree_insert(p, label, before);
@@ -993,7 +1085,7 @@ int aug_rm(struct augeas *aug, const char *path) {
 
     api_entry(aug);
 
-    p = pathx_aug_parse(aug, aug->origin, path, true);
+    p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), path, true);
     ERR_BAIL(aug);
 
     result = tree_rm(p);
@@ -1017,7 +1109,7 @@ int aug_span(struct augeas *aug, const char *path, char **filename,
 
     api_entry(aug);
 
-    p = pathx_aug_parse(aug, aug->origin, path, true);
+    p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), path, true);
     ERR_BAIL(aug);
 
     tree = pathx_first(p);
@@ -1069,7 +1161,7 @@ int tree_replace(struct augeas *aug, const char *path, struct tree *sub) {
     struct pathx *p = NULL;
     int r;
 
-    p = pathx_aug_parse(aug, aug->origin, path, true);
+    p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), path, true);
     ERR_BAIL(aug);
 
     r = tree_rm(p);
@@ -1099,10 +1191,10 @@ int aug_mv(struct augeas *aug, const char *src, const char *dst) {
     api_entry(aug);
 
     ret = -1;
-    s = pathx_aug_parse(aug, aug->origin, src, true);
+    s = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), src, true);
     ERR_BAIL(aug);
 
-    d = pathx_aug_parse(aug, aug->origin, dst, true);
+    d = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), dst, true);
     ERR_BAIL(aug);
 
     r = find_one_node(s, &ts);
@@ -1158,7 +1250,7 @@ int aug_match(const struct augeas *aug, const char *pathin, char ***matches) {
         pathin = "/*";
     }
 
-    p = pathx_aug_parse(aug, aug->origin, pathin, true);
+    p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), pathin, true);
     ERR_BAIL(aug);
 
     for (tree = pathx_first(p); tree != NULL; tree = pathx_next(p)) {
@@ -1298,7 +1390,7 @@ static int unlink_removed_files(struct augeas *aug,
             /* Unlink all files in tm */
             struct pathx *px = NULL;
             if (pathx_parse(tm, err_of_aug(aug), file_nodes, true,
-                            aug->symtab, &px) != PATHX_NOERROR) {
+                            aug->symtab, NULL, &px) != PATHX_NOERROR) {
                 result = -1;
                 continue;
             }
@@ -1366,7 +1458,7 @@ static int print_one(FILE *out, const char *path, const char *value) {
     if (r < 0)
         return -1;
     if (value != NULL) {
-        char *val = escape(value, -1);
+        char *val = escape(value, -1, STR_ESCAPES);
         r = fprintf(out, " = \"%s\"", val);
         free(val);
         if (r < 0)
@@ -1438,7 +1530,7 @@ int dump_tree(FILE *out, struct tree *tree) {
     struct pathx *p;
     int result;
 
-    if (pathx_parse(tree, NULL, "/*", true, NULL, &p) != PATHX_NOERROR)
+    if (pathx_parse(tree, NULL, "/*", true, NULL, NULL, &p) != PATHX_NOERROR)
         return -1;
 
     result = print_tree(out, p, 1);
@@ -1446,6 +1538,124 @@ int dump_tree(FILE *out, struct tree *tree) {
     return result;
 }
 
+static int to_xml_one(xmlNodePtr elem, const struct tree *tree,
+                      const char *pathin) {
+    xmlNodePtr value;
+    xmlAttrPtr prop;
+
+    prop = xmlSetProp(elem, BAD_CAST "label", BAD_CAST tree->label);
+    if (prop == NULL)
+        goto error;
+
+    if (tree->span) {
+        prop = xmlSetProp(elem, BAD_CAST "file",
+                          BAD_CAST tree->span->filename->str);
+        if (prop == NULL)
+            goto error;
+    }
+
+    if (pathin != NULL) {
+        prop = xmlSetProp(elem, BAD_CAST "path", BAD_CAST pathin);
+        if (prop == NULL)
+            goto error;
+    }
+    if (tree->value != NULL) {
+        value = xmlNewTextChild(elem, NULL, BAD_CAST "value",
+                                BAD_CAST tree->value);
+        if (value == NULL)
+            goto error;
+    }
+    return 0;
+ error:
+    return -1;
+}
+
+static int to_xml_rec(xmlNodePtr pnode, struct tree *start,
+                      const char *pathin) {
+    int r;
+    xmlNodePtr elem;
+
+    elem = xmlNewChild(pnode, NULL, BAD_CAST "node", NULL);
+    if (elem == NULL)
+        goto error;
+    r = to_xml_one(elem, start, pathin);
+    if (r < 0)
+        goto error;
+
+    list_for_each(tree, start->children) {
+        if (TREE_HIDDEN(tree))
+            continue;
+        r = to_xml_rec(elem, tree, NULL);
+        if (r < 0)
+            goto error;
+    }
+
+    return 0;
+ error:
+    return -1;
+}
+
+static int tree_to_xml(struct pathx *p, xmlNode **xml, const char *pathin) {
+    char *path = NULL;
+    struct tree *tree;
+    xmlAttrPtr expr;
+    int r;
+
+    *xml = xmlNewNode(NULL, BAD_CAST "augeas");
+    if (*xml == NULL)
+        goto error;
+    expr = xmlSetProp(*xml, BAD_CAST "match", BAD_CAST pathin);
+    if (expr == NULL)
+        goto error;
+
+    for (tree = pathx_first(p); tree != NULL; tree = pathx_next(p)) {
+        if (TREE_HIDDEN(tree))
+            continue;
+        path = path_of_tree(tree);
+        if (path == NULL)
+            goto error;
+        r = to_xml_rec(*xml, tree, path);
+        if (r < 0)
+            goto error;
+        FREE(path);
+    }
+    return 0;
+ error:
+    free(path);
+    xmlFree(*xml);
+    *xml = NULL;
+    return -1;
+}
+
+int aug_to_xml(const struct augeas *aug, const char *pathin,
+               xmlNode **xmldoc, unsigned int flags) {
+    struct pathx *p;
+    int result;
+
+    api_entry(aug);
+
+    ARG_CHECK(flags != 0, aug, "aug_to_xml: FLAGS must be 0");
+    ARG_CHECK(xmldoc == NULL, aug, "aug_to_xml: XMLDOC must be non-NULL");
+
+    if (pathin == NULL || strlen(pathin) == 1) {
+        pathin = "/*";
+    }
+
+    p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), pathin, true);
+    ERR_BAIL(aug);
+    result = tree_to_xml(p, xmldoc, pathin);
+    ERR_THROW(result < 0, aug, AUG_ENOMEM, NULL);
+    free_pathx(p);
+    api_exit(aug);
+
+    return result;
+ error:
+    if (xmldoc !=NULL)
+        *xmldoc = NULL;
+    api_exit(aug);
+    return -1;
+}
+
 int aug_print(const struct augeas *aug, FILE *out, const char *pathin) {
     struct pathx *p;
     int result;
@@ -1456,7 +1666,7 @@ int aug_print(const struct augeas *aug, FILE *out, const char *pathin) {
         pathin = "/*";
     }
 
-    p = pathx_aug_parse(aug, aug->origin, pathin, true);
+    p = pathx_aug_parse(aug, aug->origin, tree_root_ctx(aug), pathin, true);
     ERR_BAIL(aug);
 
     result = print_tree(out, p, 0);
