@@ -1,7 +1,7 @@
 /*
  * augeas.h: public headers for augeas
  *
- * Copyright (C) 2007-2010 David Lutterkort
+ * Copyright (C) 2007-2011 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,6 +21,7 @@
  */
 
 #include <stdio.h>
+#include <libxml/tree.h>
 
 #ifndef AUGEAS_H_
 #define AUGEAS_H_
@@ -48,7 +49,9 @@ enum aug_flags {
                                      what would have changed */
     AUG_NO_LOAD      = (1 << 5),  /* Do not load the tree from AUG_INIT */
     AUG_NO_MODL_AUTOLOAD = (1 << 6),
-    AUG_ENABLE_SPAN  = (1 << 7)   /* Track the span in the input of nodes */
+    AUG_ENABLE_SPAN  = (1 << 7),  /* Track the span in the input of nodes */
+    AUG_NO_ERR_CLOSE = (1 << 8)   /* Do not close automatically when
+                                     encountering error during aug_init */
 };
 
 /* Function: aug_init
@@ -62,11 +65,20 @@ enum aug_flags {
  * searched in. This is in addition to the standard load path and the
  * directories in AUGEAS_LENS_LIB
  *
- * FLAGS is a bitmask made up of values from AUG_FLAGS.
+ * FLAGS is a bitmask made up of values from AUG_FLAGS. The flag
+ * AUG_NO_ERR_CLOSE can be used to get more information on why
+ * initialization failed. If it is set in FLAGS, the caller must check that
+ * aug_error returns AUG_NOERROR before using the returned augeas handle
+ * for any other operation. If the handle reports any error, the caller
+ * should only call the aug_error functions an aug_close on this handle.
  *
  * Returns:
  * a handle to the Augeas tree upon success. If initialization fails,
- * returns NULL.
+ * returns NULL if AUG_NO_ERR_CLOSE is not set in FLAGS. If
+ * AUG_NO_ERR_CLOSE is set, might return an Augeas handle even on
+ * failure. In that case, caller must check for errors using augeas_error,
+ * and, if an error is reported, only use the handle with the aug_error
+ * functions and aug_close.
  */
 augeas *aug_init(const char *root, const char *loadpath, unsigned int flags);
 
@@ -74,7 +86,7 @@ augeas *aug_init(const char *root, const char *loadpath, unsigned int flags);
  *
  * Define a variable NAME whose value is the result of evaluating EXPR. If
  * a variable NAME already exists, its name will be replaced with the
- * result of evaluating EXPR.
+ * result of evaluating EXPR.  Context will not be applied to EXPR.
  *
  * If EXPR is NULL, the variable NAME will be removed if it is defined.
  *
@@ -248,7 +260,7 @@ int aug_match(const augeas *aug, const char *path, char ***matches);
  * 0 on success. Only files that had any changes made to them are written.
  *
  * If AUG_SAVE_NEWFILE is set in the FLAGS passed to AUG_INIT, create
- * changed files as new files with the extension ".augnew", and leave teh
+ * changed files as new files with the extension ".augnew", and leave the
  * original file unmodified.
  *
  * Otherwise, if AUG_SAVE_BACKUP is set in the FLAGS passed to AUG_INIT,
@@ -298,6 +310,19 @@ int aug_load(augeas *aug);
  */
 int aug_print(const augeas *aug, FILE *out, const char *path);
 
+/* Function: aug_to_xml
+ *
+ * Turn the Augeas tree(s) matching PATH into an XML tree XMLDOC. The
+ * parameter FLAGS is currently unused and must be set to 0.
+ *
+ * Returns:
+ * 0 on success, or a negative value on failure
+ *
+ * In case of failure, *xmldoc is set to NULL
+ */
+int aug_to_xml(const augeas *aug, const char *path, xmlNode **xmldoc,
+               unsigned int flags);
+
 /*
  * Function: aug_srun
  *
@@ -336,7 +361,8 @@ typedef enum {
     AUG_EMXFM,          /* Multiple transforms */
     AUG_ENOSPAN,        /* No span for this node */
     AUG_EMVDESC,        /* Cannot move node into its descendant */
-    AUG_ECMDRUN         /* Failed to execute command */
+    AUG_ECMDRUN,        /* Failed to execute command */
+    AUG_EBADARG         /* Invalid argument in funcion call */
 } aug_errcode_t;
 
 /* Return the error code from the last API call */
