@@ -27,23 +27,7 @@
 #include "errcode.h"
 
 #include <ctype.h>
-
-static char *cleanstr(char *path, const char sep) {
-    if (path == NULL || strlen(path) == 0)
-        return path;
-    char *e = path + strlen(path) - 1;
-    while (e >= path && (*e == sep || isspace(*e)))
-        *e-- = '\0';
-    return path;
-}
-
-static char *cleanpath(char *path) {
-    if (path == NULL || strlen(path) == 0)
-        return path;
-    if (STREQ(path, "/"))
-        return path;
-    return cleanstr(path, SEP);
-}
+#include <libxml/tree.h>
 
 /*
  * Command handling infrastructure
@@ -132,9 +116,10 @@ static const char *arg_value(const struct command *cmd, const char *name) {
 }
 
 static char *nexttoken(struct command *cmd, char **line, bool path) {
-    char *r, *s;
+    char *r, *s, *w;
     char quot = '\0';
     int nbracket = 0;
+    int nescaped = 0;
 
     s = *line;
 
@@ -144,25 +129,55 @@ static char *nexttoken(struct command *cmd, char **line, bool path) {
         s += 1;
     }
     r = s;
+    w = s;
     while (*s) {
-        if (*s == '[') nbracket += 1;
-        if (*s == ']') nbracket -= 1;
-        if (nbracket < 0) {
-            ERR_REPORT(cmd, AUG_ECMDRUN, "unmatched [");
-            return NULL;
+        if (*s == '\\') {
+            switch (*(s+1)) {
+                case '[':
+                case ']':  /* pass both literally */
+                    nescaped = 2;
+                    break;
+                case ' ':
+                case '\t':
+                    if (quot) break;
+                case '\'':
+                case '"':
+                    if (quot && quot != *(s+1)) break;
+                case '\\':
+                    nescaped = 1;
+                    s += 1;
+                    break;
+                default:
+                    ERR_REPORT(cmd, AUG_ECMDRUN, "unknown escape sequence");
+                    return NULL;
+            }
         }
-        if ((quot && *s == quot)
-            || (!quot && isblank(*s) && (!path || nbracket == 0)))
-            break;
+
+        if (nescaped == 0) {
+            if (*s == '[') nbracket += 1;
+            if (*s == ']') nbracket -= 1;
+            if (nbracket < 0) {
+                ERR_REPORT(cmd, AUG_ECMDRUN, "unmatched [");
+                return NULL;
+            }
+            if ((quot && *s == quot)
+                || (!quot && isblank(*s) && (!path || nbracket == 0)))
+                break;
+        } else {
+            nescaped -= 1;
+        }
+
+        *w = *s;
         s += 1;
+        w += 1;
     }
     if (*s == '\0' && path && nbracket > 0) {
         ERR_REPORT(cmd, AUG_ECMDRUN, "unmatched [");
         return NULL;
     }
-    if (*s)
-        *s++ = '\0';
-    *line = s;
+    while (*w && w <= s)
+        *w++ = '\0';
+    *line = w;
     return r;
 }
 
@@ -240,6 +255,7 @@ static int parseline(struct command *cmd, char *line) {
         opt->value = tok;
         curarg += 1;
         def += 1;
+        while (*line && isblank(*line)) line += 1;
     }
 
     if (curarg < narg - nopt) {
@@ -519,16 +535,27 @@ static const struct command_opt_def cmd_mv_opts[] = {
     CMD_OPT_DEF_LAST
 };
 
+static const char const cmd_mv_help[] =
+    "Move node  SRC to DST.  SRC must match  exactly one node in  "
+    "the tree.\n DST  must either  match  exactly one  node  in the  tree,  "
+    "or may  not\n exist  yet. If  DST exists  already, it  and all  its  "
+    "descendants are\n deleted.  If  DST  does  not   exist  yet,  it  and  "
+    "all  its  missing\n ancestors are created.";
+
 static const struct command_def cmd_mv_def = {
     .name = "mv",
     .opts = cmd_mv_opts,
     .handler = cmd_mv,
     .synopsis = "move a subtree",
-    .help = "Move node  SRC to DST.  SRC must match  exactly one node in  "
-    "the tree.\n DST  must either  match  exactly one  node  in the  tree,  "
-    "or may  not\n exist  yet. If  DST exists  already, it  and all  its  "
-    "descendants are\n deleted.  If  DST  does  not   exist  yet,  it  and  "
-    "all  its  missing\n ancestors are created."
+    .help = cmd_mv_help
+};
+
+static const struct command_def cmd_move_def = {
+    .name = "move",
+    .opts = cmd_mv_opts,
+    .handler = cmd_mv,
+    .synopsis = "move a subtree (alias of 'mv')",
+    .help = cmd_mv_help
 };
 
 static void cmd_set(struct command *cmd) {
@@ -621,13 +648,17 @@ static void cmd_span(struct command *cmd) {
     char *filename = NULL;
     const char *option = NULL;
 
-    if (! streqv(AUG_DISABLE, option)) {
+    if (aug_get(cmd->aug, AUGEAS_SPAN_OPTION, &option) != 1) {
+        printf("Error: option " AUGEAS_SPAN_OPTION " not found\n");
+        return;
+    }
+    if (streqv(AUG_DISABLE, option)) {
         ERR_REPORT(cmd, AUG_ECMDRUN,
                    "Span is not enabled. To enable, run the commands:\n"
                    "    set %s %s\n    rm %s\n    load\n",
                    AUGEAS_SPAN_OPTION, AUG_ENABLE, AUGEAS_FILES_TREE);
         return;
-    } else if (streqv(AUG_ENABLE, option)) {
+    } else if (! streqv(AUG_ENABLE, option)) {
         ERR_REPORT(cmd, AUG_ECMDRUN,
                    "option %s must be %s or %s\n", AUGEAS_SPAN_OPTION,
                    AUG_ENABLE, AUG_DISABLE);
@@ -646,7 +677,7 @@ static void cmd_span(struct command *cmd) {
 
 static const struct command_opt_def cmd_span_opts[] = {
     { .type = CMD_PATH, .name = "path", .optional = false,
-      .help = "node path" },
+      .help = "path matching exactly one node" },
     CMD_OPT_DEF_LAST
 };
 
@@ -654,8 +685,8 @@ static const struct command_def cmd_span_def = {
     .name = "span",
     .opts = cmd_span_opts,
     .handler = cmd_span,
-    .synopsis = "get the filename, label and value position in the text of this node",
-    .help = "get the filename, label and value position in the text of this node"
+    .synopsis = "Print information get the filename, label and value position in the text of this node",
+    .help = "Print the name of the file from which the node PATH was generated, as\n well as information about the positions in the file  corresponding to\n the label, the value, and the  entire  node. PATH must match  exactly\n one node.\n\n You need to run 'set /augeas/span enable' prior to  loading files to\n enable recording of span information. It is disabled by default."
 };
 
 static void cmd_defvar(struct command *cmd) {
@@ -793,6 +824,43 @@ static const struct command_def cmd_print_def = {
     .help = "Print entries in the tree.  If PATH is given, printing starts there,\n otherwise the whole tree is printed"
 };
 
+static void cmd_dump_xml(struct command *cmd) {
+    const char *path = arg_value(cmd, "path");
+    const char *filename = arg_value(cmd, "filename");
+    xmlNodePtr xmldoc;
+    int r;
+
+    r = aug_to_xml(cmd->aug, path, &xmldoc, 0);
+    if (r < 0)
+        ERR_REPORT(cmd, AUG_ECMDRUN,
+                   "XML export of path %s to file %s failed", path, filename);
+
+    xmlElemDump(stdout, NULL, xmldoc);
+    printf("\n");
+
+    if (filename != NULL) {
+        printf("Saving to %s\n", filename);
+    }
+
+    xmlFreeNode(xmldoc);
+}
+
+static const struct command_opt_def cmd_dump_xml_opts[] = {
+    { .type = CMD_PATH, .name = "path", .optional = true,
+      .help = "print this subtree" },
+    { .type = CMD_NONE, .name = "filename", .optional = true,
+      .help = "save to this file" },
+    CMD_OPT_DEF_LAST
+};
+
+static const struct command_def cmd_dump_xml_def = {
+    .name = "dump-xml",
+    .opts = cmd_dump_xml_opts,
+    .handler = cmd_dump_xml,
+    .synopsis = "print a subtree as XML",
+    .help = "Export entries in the tree as XML. If PATH is given, printing starts there,\n otherwise the whole tree is printed. If FILENAME is given, the XML is saved\n to the given file."
+};
+
 static void cmd_save(struct command *cmd) {
     int r;
     r = aug_save(cmd->aug);
@@ -875,13 +943,24 @@ static const struct command_opt_def cmd_ins_opts[] = {
     CMD_OPT_DEF_LAST
 };
 
+static const char const cmd_ins_help[] =
+    "Insert a new node with label LABEL right before or after "
+    "PATH into the\n tree. WHERE must be either 'before' or 'after'.";
+
 static const struct command_def cmd_ins_def = {
     .name = "ins",
     .opts = cmd_ins_opts,
     .handler = cmd_ins,
-    .synopsis = "insert new node before/after and existing node",
-    .help = "Insert a new node with label LABEL right before or after "
-    "PATH into the\n tree. WHERE must be either 'before' or 'after'."
+    .synopsis = "insert new node",
+    .help = cmd_ins_help
+};
+
+static const struct command_def cmd_insert_def = {
+    .name = "insert",
+    .opts = cmd_ins_opts,
+    .handler = cmd_ins,
+    .synopsis = "insert new node (alias of 'ins')",
+    .help = cmd_ins_help
 };
 
 static const struct command_def const *commands[] = {
@@ -891,11 +970,14 @@ static const struct command_def const *commands[] = {
     &cmd_defvar_def,
     &cmd_get_def,
     &cmd_ins_def,
+    &cmd_insert_def,
     &cmd_load_def,
     &cmd_ls_def,
     &cmd_match_def,
     &cmd_mv_def,
+    &cmd_move_def,
     &cmd_print_def,
+    &cmd_dump_xml_def,
     &cmd_rm_def,
     &cmd_save_def,
     &cmd_set_def,
