@@ -1,7 +1,7 @@
 /*
  * augtool.c:
  *
- * Copyright (C) 2007-2010 David Lutterkort
+ * Copyright (C) 2007-2011 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -91,15 +91,18 @@ static char *readline_path_generator(const char *text, int state) {
     static int current = 0;
     static char **children = NULL;
     static int nchildren = 0;
+    static char *ctx = NULL;
+
+    char *end = strrchr(text, SEP);
+    if (end != NULL)
+        end += 1;
 
     if (state == 0) {
-        char *end = strrchr(text, SEP);
         char *path;
         if (end == NULL) {
-            if ((path = strdup("/*")) == NULL)
+            if ((path = strdup("*")) == NULL)
                 return NULL;
         } else {
-            end += 1;
             CALLOC(path, end - text + 2);
             if (path == NULL)
                 return NULL;
@@ -112,13 +115,23 @@ static char *readline_path_generator(const char *text, int state) {
         free((void *) children);
         nchildren = aug_match(aug, path, &children);
         current = 0;
+
+        ctx = NULL;
+        if (path[0] != SEP)
+            aug_get(aug, AUGEAS_CONTEXT, (const char **) &ctx);
+
         free(path);
     }
 
+    if (end == NULL)
+        end = (char *) text;
+
     while (current < nchildren) {
         char *child = children[current];
         current += 1;
-        if (STREQLEN(child, text, strlen(text))) {
+
+        char *chend = strrchr(child, SEP) + 1;
+        if (STREQLEN(chend, end, strlen(end))) {
             if (child_count(child) > 0) {
                 char *c = realloc(child, strlen(child)+2);
                 if (c == NULL)
@@ -126,6 +139,19 @@ static char *readline_path_generator(const char *text, int state) {
                 child = c;
                 strcat(child, "/");
             }
+
+            /* strip off context if the user didn't give it */
+            if (ctx != NULL) {
+                char *c = realloc(child, strlen(child)-strlen(ctx)+1);
+                if (c == NULL)
+                    return NULL;
+                int ctxidx = strlen(ctx);
+                if (child[ctxidx] == SEP)
+                    ctxidx++;
+                strcpy(c, &child[ctxidx]);
+                child = c;
+            }
+
             rl_filename_completion_desired = 1;
             rl_completion_append_character = '\0';
             return child;
@@ -141,7 +167,7 @@ static char *readline_command_generator(const char *text, int state) {
     static const char *const commands[] = {
         "quit", "clear", "defnode", "defvar",
         "get", "ins", "load", "ls", "match",
-        "mv", "print", "rm", "save", "set", "setm",
+        "mv", "print", "dump-xml", "rm", "save", "set", "setm",
         "clearm", "span", "help", NULL };
 
     static int current = 0;
@@ -352,7 +378,7 @@ static void print_version_info(void) {
         goto error;
 
     fprintf(stderr, "augtool %s <http://augeas.net/>\n", version);
-    fprintf(stderr, "Copyright (C) 2009-2010 David Lutterkort\n");
+    fprintf(stderr, "Copyright (C) 2007-2011 David Lutterkort\n");
     fprintf(stderr, "License LGPLv2+: GNU LGPL version 2.1 or later\n");
     fprintf(stderr, "                 <http://www.gnu.org/licenses/lgpl-2.1.html>\n");
     fprintf(stderr, "This is free software: you are free to change and redistribute it.\n");
@@ -372,6 +398,23 @@ static int run_command(const char *line) {
     return result;
 }
 
+static void print_aug_error(void) {
+    if (aug_error(aug) == AUG_ENOMEM) {
+        fprintf(stderr, "Out of memory.\n");
+        return;
+    }
+    if (aug_error(aug) != AUG_NOERROR) {
+        fprintf(stderr, "error: %s\n", aug_error_message(aug));
+        if (aug_error_minor_message(aug) != NULL)
+            fprintf(stderr, "error: %s\n",
+                    aug_error_minor_message(aug));
+        if (aug_error_details(aug) != NULL) {
+            fputs(aug_error_details(aug), stderr);
+            fprintf(stderr, "\n");
+        }
+    }
+}
+
 static int main_loop(void) {
     char *line = NULL;
     int ret = 0;
@@ -452,18 +495,7 @@ static int main_loop(void) {
             return ret;
         if (code < 0) {
             ret = -1;
-            if (aug_error(aug) == AUG_ENOMEM) {
-                fprintf(stderr, "Out of memory.\n");
-                return -1;
-            }
-            if (aug_error(aug) != AUG_NOERROR) {
-                fprintf(stderr, "error: %s\n", aug_error_message(aug));
-                if (aug_error_minor_message(aug) != NULL)
-                    fprintf(stderr, "error: %s\n",
-                            aug_error_minor_message(aug));
-                if (aug_error_details(aug) != NULL)
-                    fprintf(stderr, "error: %s\n", aug_error_details(aug));
-            }
+            print_aug_error();
         }
     }
 }
@@ -483,7 +515,7 @@ static int run_args(int argc, char **argv) {
     }
     code = run_command(line);
     free(line);
-    if (code == 0 && auto_save)
+    if (code >= 0 && auto_save)
         code = run_command("save");
     return (code == 0 || code == -2) ? 0 : -1;
 }
@@ -495,9 +527,11 @@ int main(int argc, char **argv) {
 
     parse_opts(argc, argv);
 
-    aug = aug_init(root, loadpath, flags);
-    if (aug == NULL) {
+    aug = aug_init(root, loadpath, flags|AUG_NO_ERR_CLOSE);
+    if (aug == NULL || aug_error(aug) != AUG_NOERROR) {
         fprintf(stderr, "Failed to initialize Augeas\n");
+        if (aug != NULL)
+            print_aug_error();
         exit(EXIT_FAILURE);
     }
     if (print_version) {
