@@ -1,7 +1,7 @@
 /*
  * builtin.c: builtin primitives
  *
- * Copyright (C) 2007-2010 David Lutterkort
+ * Copyright (C) 2007-2011 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -163,7 +163,7 @@ static struct value *pathx_parse_glue(struct info *info, struct value *tree,
     assert(tree->tag == V_TREE);
 
     if (pathx_parse(tree->origin, info->error, path->string->str, true,
-                    NULL, p) != PATHX_NOERROR) {
+                    NULL, NULL, p) != PATHX_NOERROR) {
         return make_pathx_exn(ref(info), *p);
     } else {
         return NULL;
@@ -533,6 +533,37 @@ static struct value *lns_fmt_atype(struct info *info, struct value *l) {
     return result;
 }
 
+/* V_REGEXP -> V_STRING -> V_STRING */
+static struct value *rx_match(struct info *info,
+                              struct value *rx, struct value *s) {
+    struct value *result = NULL;
+    const char *str = s->string->str;
+    struct re_registers regs;
+    int r;
+
+    MEMZERO(&regs, 1);
+    r = regexp_match(rx->regexp, str, strlen(str), 0, &regs);
+    if (r < -1) {
+        result =
+            make_exn_value(ref(info), "regexp match failed (internal error)");
+    } else {
+        char *match = NULL;
+        if (r == -1) {
+            /* No match */
+            match = strdup("");
+        } else {
+            match = strndup(str + regs.start[0], regs.end[0] - regs.start[0]);
+        }
+        if (match == NULL) {
+            result = exn_error();
+        } else {
+            result = make_value(V_STRING, ref(info));
+            result->string = make_string(match);
+        }
+    }
+    return result;
+}
+
 struct module *builtin_init(struct error *error) {
     struct module *modl = module_create("Builtin");
     int r;
@@ -586,6 +617,10 @@ struct module *builtin_init(struct error *error) {
     DEFINE_NATIVE(modl, "lens_format_atype", 1, lns_fmt_atype,
                   T_LENS, T_STRING);
 
+    /* Regexp matching */
+    DEFINE_NATIVE(modl, "regexp_match", 2, rx_match, T_REGEXP, T_STRING,
+                  T_STRING);
+
     /* System functions */
     struct module *sys = module_create("Sys");
     modl->next = sys;
