@@ -1,7 +1,7 @@
 /*
  * errcode.h: internal interface for error reporting
  *
- * Copyright (C) 2009-2010 David Lutterkort
+ * Copyright (C) 2009-2011 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -79,6 +79,14 @@ void reset_error(struct error *err);
         }                                               \
     } while(0)
 
+#define ARG_CHECK(cond, obj, fmt ...)                           \
+    do {                                                        \
+        if (cond) {                                             \
+            report_error((obj)->error, AUG_EBADARG, ## fmt);    \
+            goto error;                                         \
+        }                                                       \
+    } while(0)
+
 /* A variant of assert that uses our error reporting infrastructure
  * instead of aborting
  */
