@@ -1,7 +1,7 @@
 /*
  * parser.c: parse a configuration file according to a grammar
  *
- * Copyright (C) 2007-2010 David Lutterkort
+ * Copyright (C) 2007-2011 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -243,7 +243,7 @@ static void get_expected_error(struct state *state, struct lens *l) {
     for (p = word; *p != '\0' && *p != '\n'; p++);
     *p = '\0';
 
-    pat = escape(l->ctype->pattern->str, -1);
+    pat = escape(l->ctype->pattern->str, -1, NULL);
     get_error(state, l, "expected %s at '%s'", pat, word);
     free(pat);
 }
