@@ -1,7 +1,7 @@
 /*
  * internal.h: Useful definitions
  *
- * Copyright (C) 2007-2010 David Lutterkort
+ * Copyright (C) 2007-2011 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -81,6 +81,10 @@
 #define AUGEAS_COPY_IF_RENAME_FAILS \
     AUGEAS_META_SAVE_MODE "/copy_if_rename_fails"
 
+/* Define: AUGEAS_CONTEXT
+ * Context prepended to all non-absolute paths */
+#define AUGEAS_CONTEXT AUGEAS_META_TREE "/context"
+
 /* A hierarchy where we record certain 'events', e.g. which tree
  * nodes actually gotsaved into files */
 #define AUGEAS_EVENTS AUGEAS_META_TREE "/events"
@@ -119,6 +123,9 @@
 #define AUG_ENABLE "enable"
 #define AUG_DISABLE "disable"
 
+/* default value for the relative path context */
+#define AUG_CONTEXT_DEFAULT "/files"
+
 #ifdef __GNUC__
 
 #ifndef __GNUC_PREREQ
@@ -253,10 +260,14 @@ int pathjoin(char **path, int nseg, ...);
  * Escape nonprintable characters within TEXT, similar to how it's done in
  * C string literals. Caller must free the returned string.
  */
-char *escape(const char *text, int cnt);
+char *escape(const char *text, int cnt, const char *extra);
 
 /* Function: unescape */
-char *unescape(const char *s, int len);
+char *unescape(const char *s, int len, const char *extra);
+
+/* Extra characters to be escaped in strings and regexps respectively */
+#define STR_ESCAPES "\"\\"
+#define RX_ESCAPES  "/\\"
 
 /* Function: print_chars */
 int print_chars(FILE *out, const char *text, int cnt);
@@ -286,6 +297,9 @@ int xstrtoint64(char const *s, int base, int64_t *result);
 /* Calculate line and column number of character POS in TEXT */
 void calc_line_ofs(const char *text, size_t pos, size_t *line, size_t *ofs);
 
+/* Cleans path from user, removing trailing slashes and whitespace */
+char *cleanpath(char *path);
+
 /* Take the first LEN characters from the regexp *U and expand any
  * character ranges in it. The expanded regexp, if expansion is necessary,
  * is in U, and the old string is freed. If expansion is not needed or an
@@ -420,6 +434,12 @@ struct tree *tree_find(struct augeas *aug, const char *path);
  * Returns the node or NULL on error
  */
 struct tree *tree_find_cr(struct augeas *aug, const char *path);
+/* Find the node at the path stored in AUGEAS_CONTEXT, i.e. the root context
+ * node for relative paths.
+ * Errors: EMMATCH - more than one node matches PATH
+ *         ENOMEM  - allocation error
+ */
+struct tree *tree_root_ctx(const struct augeas *aug);
 
 /* Struct: memstream
  * Wrappers to simulate OPEN_MEMSTREAM where that's not available. The
@@ -485,7 +505,8 @@ struct pathx_symtab;
 const char *pathx_error(struct pathx *pathx, const char **txt, int *pos);
 
 /* Parse a path expression PATH rooted at TREE, which is a node somewhere
- * in AUG->ORIGIN. If TREE is NULL, AUG->ORIGIN is used.
+ * in AUG->ORIGIN. If TREE is NULL, AUG->ORIGIN is used. If ROOT_CTX is not
+ * NULL and the PATH isn't absolute then it will be rooted at ROOT_CTX.
  *
  * Use this function rather than PATHX_PARSE for path expressions inside
  * the tree in AUG->ORIGIN.
@@ -498,6 +519,7 @@ const char *pathx_error(struct pathx *pathx, const char **txt, int *pos);
  */
 struct pathx *pathx_aug_parse(const struct augeas *aug,
                               struct tree *tree,
+                              struct tree *root_ctx,
                               const char *path, bool need_nodeset);
 
 /* Parse the string PATH into a path expression PX that will be evaluated
@@ -513,6 +535,7 @@ int pathx_parse(const struct tree *origin,
                 const char *path,
                 bool need_nodeset,
                 struct pathx_symtab *symtab,
+                struct tree *root_ctx,
                 struct pathx **px);
 /* Return the error struct that was passed into pathx_parse */
 struct error *err_of_pathx(struct pathx *px);
