@@ -1,10 +1,9 @@
-
-/* A Bison parser, made by GNU Bison 2.4.1.  */
+/* A Bison parser, made by GNU Bison 2.4.3.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
    
-      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
-   Free Software Foundation, Inc.
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
+   2009, 2010 Free Software Foundation, Inc.
    
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,7 +45,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.4.1"
+#define YYBISON_VERSION "2.4.3"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -106,8 +105,8 @@ typedef struct info YYLTYPE;
 /* The lack of reference counting on filename is intentional */
 # define YYLLOC_DEFAULT(Current, Rhs, N)                                \
   do {                                                                  \
-    (Current).filename = augl_get_extra(scanner)->filename;             \
-    (Current).error = augl_get_extra(scanner)->error;                   \
+    (Current).filename = augl_get_info(scanner)->filename;              \
+    (Current).error = augl_get_info(scanner)->error;                    \
     if (YYID (N)) {                                                     \
         (Current).first_line   = YYRHSLOC (Rhs, 1).first_line;          \
         (Current).first_column = YYRHSLOC (Rhs, 1).first_column;        \
@@ -122,9 +121,8 @@ typedef struct info YYLTYPE;
   } while (0)
 
 
-
 /* Line 189 of yacc.c  */
-#line 128 "parser.c"
+#line 126 "parser.c"
 
 /* Enabling traces.  */
 #ifndef YYDEBUG
@@ -199,7 +197,7 @@ typedef union YYSTYPE
 {
 
 /* Line 214 of yacc.c  */
-#line 78 "parser.y"
+#line 89 "parser.y"
 
   struct term    *term;
   struct type    *type;
@@ -216,7 +214,7 @@ typedef union YYSTYPE
 
 
 /* Line 214 of yacc.c  */
-#line 220 "parser.c"
+#line 218 "parser.c"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -236,19 +234,38 @@ typedef struct YYLTYPE
 # define YYLTYPE_IS_TRIVIAL 1
 #endif
 
+/* "%code provides" blocks.  */
+
+/* Line 261 of yacc.c  */
+#line 46 "parser.y"
+
+#include "info.h"
+
+/* Track custom scanner state */
+struct state {
+  struct info *info;
+  unsigned int comment_depth;
+};
+
+
+
+
+/* Line 261 of yacc.c  */
+#line 255 "parser.c"
 
 /* Copy the second part of user declarations.  */
 
 /* Line 264 of yacc.c  */
-#line 103 "parser.y"
+#line 114 "parser.y"
 
 /* Lexer */
 extern int augl_lex (YYSTYPE * yylval_param,struct info * yylloc_param ,yyscan_t yyscanner);
- int augl_init_lexer(struct info *info, yyscan_t * scanner);
+int augl_init_lexer(struct state *state, yyscan_t * scanner);
+void augl_close_lexer(yyscan_t *scanner);
 int augl_lex_destroy (yyscan_t yyscanner );
 int augl_get_lineno (yyscan_t yyscanner );
 int augl_get_column  (yyscan_t yyscanner);
-struct info *augl_get_extra (yyscan_t yyscanner );
+struct info *augl_get_info(yyscan_t yyscanner);
 char *augl_get_text (yyscan_t yyscanner );
 
 static void augl_error(struct info *locp, struct term **term,
@@ -304,7 +321,7 @@ static void augl_error(struct info *locp, struct term **term,
 
 
 /* Line 264 of yacc.c  */
-#line 308 "parser.c"
+#line 325 "parser.c"
 
 #ifdef short
 # undef short
@@ -354,7 +371,7 @@ typedef short int yytype_int16;
 #define YYSIZE_MAXIMUM ((YYSIZE_T) -1)
 
 #ifndef YY_
-# if YYENABLE_NLS
+# if defined YYENABLE_NLS && YYENABLE_NLS
 #  if ENABLE_NLS
 #   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
 #   define YY_(msgid) dgettext ("bison-runtime", msgid)
@@ -613,13 +630,13 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   167,   167,   170,   173,   175,   180,   185,   190,   196,
-     200,   202,   205,   207,   211,   216,   218,   220,   223,   225,
-     227,   230,   232,   235,   237,   240,   242,   245,   247,   249,
-     251,   253,   255,   258,   260,   263,   265,   267,   270,   272,
-     274,   276,   279,   282,   284,   287,   289,   291,   294,   296,
-     299,   301,   303,   305,   308,   310,   313,   318,   320,   324,
-     328,   330
+       0,   179,   179,   182,   185,   187,   192,   197,   202,   208,
+     212,   214,   217,   219,   223,   228,   230,   232,   235,   237,
+     239,   242,   244,   247,   249,   252,   254,   257,   259,   261,
+     263,   265,   267,   270,   272,   275,   277,   279,   282,   284,
+     286,   288,   291,   294,   296,   299,   301,   303,   306,   308,
+     311,   313,   315,   317,   320,   322,   325,   330,   332,   336,
+     340,   342
 };
 #endif
 
@@ -799,9 +816,18 @@ static const yytype_uint8 yystos[] =
 
 /* Like YYERROR except do call yyerror.  This remains here temporarily
    to ease the transition to the new meaning of YYERROR, for GCC.
-   Once GCC version 2 has supplanted version 1, this can go.  */
+   Once GCC version 2 has supplanted version 1, this can go.  However,
+   YYFAIL appears to be in use.  Nevertheless, it is formally deprecated
+   in Bison 2.4.2's NEWS entry, where a plan to phase it out is
+   discussed.  */
 
 #define YYFAIL		goto yyerrlab
+#if defined YYFAIL
+  /* This is here to suppress warnings from the GCC cpp's
+     -Wunused-macros.  Normally we don't worry about that warning, but
+     some users do, and we want to make it easy for users to remove
+     YYFAIL uses, which will produce warnings from Bison 2.5.  */
+#endif
 
 #define YYRECOVERING()  (!!yyerrstatus)
 
@@ -858,7 +884,7 @@ while (YYID (0))
    we won't break user code: when these are the locations we know.  */
 
 #ifndef YY_LOCATION_PRINT
-# if YYLTYPE_IS_TRIVIAL
+# if defined YYLTYPE_IS_TRIVIAL && YYLTYPE_IS_TRIVIAL
 #  define YY_LOCATION_PRINT(File, Loc)			\
      fprintf (File, "%d.%d-%d.%d",			\
 	      (Loc).first_line, (Loc).first_column,	\
@@ -1407,7 +1433,7 @@ YYLTYPE yylloc;
     YYLTYPE *yylsp;
 
     /* The locations where the error started and ended.  */
-    YYLTYPE yyerror_range[2];
+    YYLTYPE yyerror_range[3];
 
     YYSIZE_T yystacksize;
 
@@ -1454,7 +1480,7 @@ YYLTYPE yylloc;
   yyvsp = yyvs;
   yylsp = yyls;
 
-#if YYLTYPE_IS_TRIVIAL
+#if defined YYLTYPE_IS_TRIVIAL && YYLTYPE_IS_TRIVIAL
   /* Initialize the default location before parsing starts.  */
   yylloc.first_line   = yylloc.last_line   = 1;
   yylloc.first_column = yylloc.last_column = 1;
@@ -1462,19 +1488,19 @@ YYLTYPE yylloc;
 
 /* User initialization code.  */
 
-/* Line 1242 of yacc.c  */
-#line 55 "parser.y"
+/* Line 1251 of yacc.c  */
+#line 66 "parser.y"
 {
   yylloc.first_line   = 1;
   yylloc.first_column = 0;
   yylloc.last_line    = 1;
   yylloc.last_column  = 0;
-  yylloc.filename     = augl_get_extra(scanner)->filename;
-  yylloc.error        = augl_get_extra(scanner)->error;
+  yylloc.filename     = augl_get_info(scanner)->filename;
+  yylloc.error        = augl_get_info(scanner)->error;
 }
 
-/* Line 1242 of yacc.c  */
-#line 1478 "parser.c"
+/* Line 1251 of yacc.c  */
+#line 1504 "parser.c"
   yylsp[0] = yylloc;
 
   goto yysetstate;
@@ -1661,29 +1687,29 @@ yyreduce:
     {
         case 2:
 
-/* Line 1455 of yacc.c  */
-#line 168 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 180 "parser.y"
     { (*term) = make_module((yyvsp[(2) - (5)].string), (yyvsp[(4) - (5)].string), (yyvsp[(5) - (5)].term), &(yylsp[(1) - (5)])); }
     break;
 
   case 3:
 
-/* Line 1455 of yacc.c  */
-#line 171 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 183 "parser.y"
     { (yyval.string) = (yyvsp[(2) - (2)].string); }
     break;
 
   case 4:
 
-/* Line 1455 of yacc.c  */
-#line 173 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 185 "parser.y"
     { (yyval.string) = NULL; }
     break;
 
   case 5:
 
-/* Line 1455 of yacc.c  */
-#line 176 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 188 "parser.y"
     {
          LOC_MERGE((yylsp[(1) - (6)]), (yylsp[(1) - (6)]), (yylsp[(5) - (6)]));
          (yyval.term) = make_bind((yyvsp[(2) - (6)].string), (yyvsp[(3) - (6)].term), (yyvsp[(5) - (6)].term), (yyvsp[(6) - (6)].term), &(yylsp[(1) - (6)]));
@@ -1692,8 +1718,8 @@ yyreduce:
 
   case 6:
 
-/* Line 1455 of yacc.c  */
-#line 181 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 193 "parser.y"
     {
          LOC_MERGE((yylsp[(1) - (5)]), (yylsp[(1) - (5)]), (yylsp[(4) - (5)]));
          (yyval.term) = make_bind_rec((yyvsp[(2) - (5)].string), (yyvsp[(4) - (5)].term), (yyvsp[(5) - (5)].term), &(yylsp[(1) - (5)]));
@@ -1702,8 +1728,8 @@ yyreduce:
 
   case 7:
 
-/* Line 1455 of yacc.c  */
-#line 186 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 198 "parser.y"
     {
          LOC_MERGE((yylsp[(1) - (5)]), (yylsp[(1) - (5)]), (yylsp[(4) - (5)]));
          (yyval.term) = make_test((yyvsp[(2) - (5)].term), (yyvsp[(4) - (5)].term), TR_CHECK, (yyvsp[(5) - (5)].term), &(yylsp[(1) - (5)]));
@@ -1712,8 +1738,8 @@ yyreduce:
 
   case 8:
 
-/* Line 1455 of yacc.c  */
-#line 191 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 203 "parser.y"
     {
          LOC_MERGE((yylsp[(1) - (5)]), (yylsp[(1) - (5)]), (yylsp[(4) - (5)]));
          (yyval.term) = make_test((yyvsp[(2) - (5)].term), NULL, (yyvsp[(4) - (5)].intval), (yyvsp[(5) - (5)].term), &(yylsp[(1) - (5)]));
@@ -1722,43 +1748,43 @@ yyreduce:
 
   case 9:
 
-/* Line 1455 of yacc.c  */
-#line 196 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 208 "parser.y"
     { (yyval.term) = NULL; }
     break;
 
   case 10:
 
-/* Line 1455 of yacc.c  */
-#line 201 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 213 "parser.y"
     { (yyval.term) = make_get_test((yyvsp[(1) - (3)].term), (yyvsp[(3) - (3)].term), &(yyloc)); }
     break;
 
   case 11:
 
-/* Line 1455 of yacc.c  */
-#line 203 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 215 "parser.y"
     { (yyval.term) = make_put_test((yyvsp[(1) - (5)].term), (yyvsp[(3) - (5)].term), (yyvsp[(5) - (5)].term), &(yyloc)); }
     break;
 
   case 12:
 
-/* Line 1455 of yacc.c  */
-#line 206 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 218 "parser.y"
     { (yyval.intval) = TR_PRINT; }
     break;
 
   case 13:
 
-/* Line 1455 of yacc.c  */
-#line 208 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 220 "parser.y"
     { (yyval.intval) = TR_EXN; }
     break;
 
   case 14:
 
-/* Line 1455 of yacc.c  */
-#line 212 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 224 "parser.y"
     {
        LOC_MERGE((yylsp[(1) - (7)]), (yylsp[(1) - (7)]), (yylsp[(6) - (7)]));
        (yyval.term) = make_let((yyvsp[(2) - (7)].string), (yyvsp[(3) - (7)].term), (yyvsp[(5) - (7)].term), (yyvsp[(7) - (7)].term), &(yylsp[(1) - (7)]));
@@ -1767,288 +1793,288 @@ yyreduce:
 
   case 16:
 
-/* Line 1455 of yacc.c  */
-#line 219 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 231 "parser.y"
     { (yyval.term) = make_binop(A_COMPOSE, (yyvsp[(1) - (3)].term), (yyvsp[(3) - (3)].term), &(yyloc)); }
     break;
 
   case 17:
 
-/* Line 1455 of yacc.c  */
-#line 221 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 233 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 18:
 
-/* Line 1455 of yacc.c  */
-#line 224 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 236 "parser.y"
     { (yyval.term) = make_binop(A_UNION, (yyvsp[(1) - (3)].term), (yyvsp[(3) - (3)].term), &(yyloc)); }
     break;
 
   case 19:
 
-/* Line 1455 of yacc.c  */
-#line 226 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 238 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 20:
 
-/* Line 1455 of yacc.c  */
-#line 228 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 240 "parser.y"
     { (yyval.term) = make_tree_value((yyvsp[(1) - (1)].tree), &(yylsp[(1) - (1)])); }
     break;
 
   case 21:
 
-/* Line 1455 of yacc.c  */
-#line 231 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 243 "parser.y"
     { (yyval.term) = make_binop(A_MINUS, (yyvsp[(1) - (3)].term), (yyvsp[(3) - (3)].term), &(yyloc)); }
     break;
 
   case 22:
 
-/* Line 1455 of yacc.c  */
-#line 233 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 245 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 23:
 
-/* Line 1455 of yacc.c  */
-#line 236 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 248 "parser.y"
     { (yyval.term) = make_binop(A_CONCAT, (yyvsp[(1) - (3)].term), (yyvsp[(3) - (3)].term), &(yyloc)); }
     break;
 
   case 24:
 
-/* Line 1455 of yacc.c  */
-#line 238 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 250 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 25:
 
-/* Line 1455 of yacc.c  */
-#line 241 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 253 "parser.y"
     { (yyval.term) = make_binop(A_APP, (yyvsp[(1) - (2)].term), (yyvsp[(2) - (2)].term), &(yyloc)); }
     break;
 
   case 26:
 
-/* Line 1455 of yacc.c  */
-#line 243 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 255 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 27:
 
-/* Line 1455 of yacc.c  */
-#line 246 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 258 "parser.y"
     { (yyval.term) = make_ident((yyvsp[(1) - (1)].string), &(yylsp[(1) - (1)])); }
     break;
 
   case 28:
 
-/* Line 1455 of yacc.c  */
-#line 248 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 260 "parser.y"
     { (yyval.term) = make_string_term((yyvsp[(1) - (1)].string), &(yylsp[(1) - (1)])); }
     break;
 
   case 29:
 
-/* Line 1455 of yacc.c  */
-#line 250 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 262 "parser.y"
     { (yyval.term) = make_regexp_term((yyvsp[(1) - (1)].regexp).pattern, (yyvsp[(1) - (1)].regexp).nocase, &(yylsp[(1) - (1)])); }
     break;
 
   case 30:
 
-/* Line 1455 of yacc.c  */
-#line 252 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 264 "parser.y"
     { (yyval.term) = (yyvsp[(2) - (3)].term); }
     break;
 
   case 31:
 
-/* Line 1455 of yacc.c  */
-#line 254 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 266 "parser.y"
     { (yyval.term) = make_unop(A_BRACKET, (yyvsp[(2) - (3)].term), &(yyloc)); }
     break;
 
   case 32:
 
-/* Line 1455 of yacc.c  */
-#line 256 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 268 "parser.y"
     { (yyval.term) = make_unit_term(&(yyloc)); }
     break;
 
   case 33:
 
-/* Line 1455 of yacc.c  */
-#line 259 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 271 "parser.y"
     { (yyval.term) = make_rep((yyvsp[(1) - (2)].term), (yyvsp[(2) - (2)].quant), &(yyloc)); }
     break;
 
   case 34:
 
-/* Line 1455 of yacc.c  */
-#line 261 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 273 "parser.y"
     { (yyval.term) = (yyvsp[(1) - (1)].term); }
     break;
 
   case 35:
 
-/* Line 1455 of yacc.c  */
-#line 264 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 276 "parser.y"
     { (yyval.quant) = Q_STAR; }
     break;
 
   case 36:
 
-/* Line 1455 of yacc.c  */
-#line 266 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 278 "parser.y"
     { (yyval.quant) = Q_PLUS; }
     break;
 
   case 37:
 
-/* Line 1455 of yacc.c  */
-#line 268 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 280 "parser.y"
     { (yyval.quant) = Q_MAYBE; }
     break;
 
   case 38:
 
-/* Line 1455 of yacc.c  */
-#line 271 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 283 "parser.y"
     { (yyval.string) = (yyvsp[(1) - (1)].string); }
     break;
 
   case 39:
 
-/* Line 1455 of yacc.c  */
-#line 273 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 285 "parser.y"
     { (yyval.string) = (yyvsp[(1) - (1)].string); }
     break;
 
   case 40:
 
-/* Line 1455 of yacc.c  */
-#line 275 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 287 "parser.y"
     { (yyval.string) = strdup("get"); }
     break;
 
   case 41:
 
-/* Line 1455 of yacc.c  */
-#line 277 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 289 "parser.y"
     { (yyval.string) = strdup("put"); }
     break;
 
   case 42:
 
-/* Line 1455 of yacc.c  */
-#line 280 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 292 "parser.y"
     { (yyval.term) = (yyvsp[(2) - (2)].term); list_cons((yyval.term), (yyvsp[(1) - (2)].term)); }
     break;
 
   case 43:
 
-/* Line 1455 of yacc.c  */
-#line 282 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 294 "parser.y"
     { (yyval.term) = NULL; }
     break;
 
   case 44:
 
-/* Line 1455 of yacc.c  */
-#line 285 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 297 "parser.y"
     { (yyval.term) = make_param((yyvsp[(2) - (5)].string), (yyvsp[(4) - (5)].type), clone_info(&(yylsp[(1) - (5)]))); }
     break;
 
   case 45:
 
-/* Line 1455 of yacc.c  */
-#line 288 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 300 "parser.y"
     { (yyval.string) = (yyvsp[(1) - (1)].string); }
     break;
 
   case 46:
 
-/* Line 1455 of yacc.c  */
-#line 290 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 302 "parser.y"
     { (yyval.string) = strdup("get"); }
     break;
 
   case 47:
 
-/* Line 1455 of yacc.c  */
-#line 292 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 304 "parser.y"
     { (yyval.string) = strdup("put"); }
     break;
 
   case 48:
 
-/* Line 1455 of yacc.c  */
-#line 295 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 307 "parser.y"
     { (yyval.type) = make_arrow_type((yyvsp[(1) - (3)].type), (yyvsp[(3) - (3)].type)); }
     break;
 
   case 49:
 
-/* Line 1455 of yacc.c  */
-#line 297 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 309 "parser.y"
     { (yyval.type) = (yyvsp[(1) - (1)].type); }
     break;
 
   case 50:
 
-/* Line 1455 of yacc.c  */
-#line 300 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 312 "parser.y"
     { (yyval.type) = make_base_type(T_STRING); }
     break;
 
   case 51:
 
-/* Line 1455 of yacc.c  */
-#line 302 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 314 "parser.y"
     { (yyval.type) = make_base_type(T_REGEXP); }
     break;
 
   case 52:
 
-/* Line 1455 of yacc.c  */
-#line 304 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 316 "parser.y"
     { (yyval.type) = make_base_type(T_LENS); }
     break;
 
   case 53:
 
-/* Line 1455 of yacc.c  */
-#line 306 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 318 "parser.y"
     { (yyval.type) = (yyvsp[(2) - (3)].type); }
     break;
 
   case 54:
 
-/* Line 1455 of yacc.c  */
-#line 309 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 321 "parser.y"
     { (yyval.tree) = tree_concat((yyvsp[(1) - (4)].tree), (yyvsp[(3) - (4)].tree)); }
     break;
 
   case 55:
 
-/* Line 1455 of yacc.c  */
-#line 311 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 323 "parser.y"
     { (yyval.tree) = tree_concat((yyvsp[(2) - (3)].tree), NULL); }
     break;
 
   case 56:
 
-/* Line 1455 of yacc.c  */
-#line 314 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 326 "parser.y"
     {
               (yyval.tree) = tree_concat((yyvsp[(1) - (4)].tree), (yyvsp[(3) - (4)].tree));
             }
@@ -2056,15 +2082,15 @@ yyreduce:
 
   case 57:
 
-/* Line 1455 of yacc.c  */
-#line 318 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 330 "parser.y"
     { (yyval.tree) = NULL; }
     break;
 
   case 58:
 
-/* Line 1455 of yacc.c  */
-#line 321 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 333 "parser.y"
     {
                (yyval.tree) = make_tree((yyvsp[(1) - (2)].string), NULL, NULL, (yyvsp[(2) - (2)].tree));
              }
@@ -2072,8 +2098,8 @@ yyreduce:
 
   case 59:
 
-/* Line 1455 of yacc.c  */
-#line 325 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 337 "parser.y"
     {
                (yyval.tree) = make_tree((yyvsp[(1) - (4)].string), (yyvsp[(3) - (4)].string), NULL, (yyvsp[(4) - (4)].tree));
              }
@@ -2081,15 +2107,15 @@ yyreduce:
 
   case 61:
 
-/* Line 1455 of yacc.c  */
-#line 330 "parser.y"
+/* Line 1464 of yacc.c  */
+#line 342 "parser.y"
     { (yyval.string) = NULL; }
     break;
 
 
 
-/* Line 1455 of yacc.c  */
-#line 2093 "parser.c"
+/* Line 1464 of yacc.c  */
+#line 2119 "parser.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2161,7 +2187,7 @@ yyerrlab:
 #endif
     }
 
-  yyerror_range[0] = yylloc;
+  yyerror_range[1] = yylloc;
 
   if (yyerrstatus == 3)
     {
@@ -2198,7 +2224,7 @@ yyerrorlab:
   if (/*CONSTCOND*/ 0)
      goto yyerrorlab;
 
-  yyerror_range[0] = yylsp[1-yylen];
+  yyerror_range[1] = yylsp[1-yylen];
   /* Do not reclaim the symbols of the rule which action triggered
      this YYERROR.  */
   YYPOPSTACK (yylen);
@@ -2232,7 +2258,7 @@ yyerrlab1:
       if (yyssp == yyss)
 	YYABORT;
 
-      yyerror_range[0] = *yylsp;
+      yyerror_range[1] = *yylsp;
       yydestruct ("Error: popping",
 		  yystos[yystate], yyvsp, yylsp, term, scanner);
       YYPOPSTACK (1);
@@ -2242,10 +2268,10 @@ yyerrlab1:
 
   *++yyvsp = yylval;
 
-  yyerror_range[1] = yylloc;
+  yyerror_range[2] = yylloc;
   /* Using YYLLOC is tempting, but would change the location of
      the lookahead.  YYLOC is available though.  */
-  YYLLOC_DEFAULT (yyloc, (yyerror_range - 1), 2);
+  YYLLOC_DEFAULT (yyloc, yyerror_range, 2);
   *++yylsp = yyloc;
 
   /* Shift the error token.  */
@@ -2307,13 +2333,14 @@ yyreturn:
 
 
 
-/* Line 1675 of yacc.c  */
-#line 331 "parser.y"
+/* Line 1684 of yacc.c  */
+#line 343 "parser.y"
 
 
 int augl_parse_file(struct augeas *aug, const char *name,
                     struct term **term) {
-  yyscan_t       scanner;
+  yyscan_t          scanner;
+  struct state      state;
   struct string  *sname = NULL;
   struct info    info;
   int result = -1;
@@ -2331,13 +2358,19 @@ int augl_parse_file(struct augeas *aug, const char *name,
   info.ref = UINT_MAX;
   info.filename = sname;
   info.error = aug->error;
-  if (augl_init_lexer(&info, &scanner) < 0) {
+
+  MEMZERO(&state, 1);
+  state.info = &info;
+  state.comment_depth = 0;
+
+  if (augl_init_lexer(&state, &scanner) < 0) {
     augl_error(&info, term, NULL, "file not found");
     goto error;
   }
 
   yydebug = getenv("YYDEBUG") != NULL;
   r = augl_parse(term, scanner);
+  augl_close_lexer(scanner);
   augl_lex_destroy(scanner);
   if (r == 1) {
     augl_error(&info, term, NULL, "syntax error");
@@ -2598,8 +2631,8 @@ void augl_error(struct info *locp,
     info.first_column = augl_get_column(scanner);
     info.last_line    = augl_get_lineno(scanner);
     info.last_column  = augl_get_column(scanner);
-    info.filename     = augl_get_extra(scanner)->filename;
-    info.error        = augl_get_extra(scanner)->error;
+    info.filename     = augl_get_info(scanner)->filename;
+    info.error        = augl_get_info(scanner)->error;
   } else if (*term != NULL && (*term)->info != NULL) {
     memcpy(&info, (*term)->info, sizeof(info));
   } else {
