@@ -1,7 +1,7 @@
 /*
  * pathx.c: handling path expressions
  *
- * Copyright (C) 2007-2010 David Lutterkort
+ * Copyright (C) 2007-2011 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -134,6 +134,9 @@ struct step {
     struct pred *predicates;
 };
 
+/* Initialise the root nodeset with the first step */
+static struct tree *step_root(struct step *step, struct tree *ctx,
+                              struct tree *root_ctx);
 /* Iteration over the nodes on a step, ignoring the predicates */
 static struct tree *step_first(struct step *step, struct tree *ctx);
 static struct tree *step_next(struct step *step, struct tree *ctx,
@@ -221,6 +224,8 @@ struct state {
     uint            ctx_pos;
     uint            ctx_len;
 
+    struct tree *root_ctx; /* Root context for relative paths */
+
     /* A table of all values. The table is dynamically reallocated, i.e.
      * pointers to struct value should not be used across calls that
      * might allocate new values
@@ -279,9 +284,11 @@ static void func_count(struct state *state);
 static void func_label(struct state *state);
 static void func_regexp(struct state *state);
 static void func_glob(struct state *state);
+static void func_int(struct state *state);
 
 static const enum type const arg_types_nodeset[] = { T_NODESET };
 static const enum type const arg_types_string[] = { T_STRING };
+static const enum type const arg_types_bool[] = { T_BOOLEAN };
 
 static const struct func builtin_funcs[] = {
     { .name = "last", .arity = 0, .type = T_NUMBER, .arg_types = NULL,
@@ -304,7 +311,13 @@ static const struct func builtin_funcs[] = {
       .impl = func_glob },
     { .name = "glob", .arity = 1, .type = T_REGEXP,
       .arg_types = arg_types_nodeset,
-      .impl = func_glob }
+      .impl = func_glob },
+    { .name = "int", .arity = 1, .type = T_NUMBER,
+      .arg_types = arg_types_string, .impl = func_int },
+    { .name = "int", .arity = 1, .type = T_NUMBER,
+      .arg_types = arg_types_nodeset, .impl = func_int },
+    { .name = "int", .arity = 1, .type = T_NUMBER,
+      .arg_types = arg_types_bool, .impl = func_int }
 };
 
 #define RET_ON_ERROR                                                    \
@@ -642,6 +655,39 @@ static void func_label(struct state *state) {
     push_value(vind, state);
 }
 
+static void func_int(struct state *state) {
+    value_ind_t vind = make_value(T_NUMBER, state);
+    int64_t i = -1;
+    RET_ON_ERROR;
+
+    struct value *v = pop_value(state);
+    if (v->tag == T_BOOLEAN) {
+        i = v->boolval;
+    } else {
+        const char *s = NULL;
+        if (v->tag == T_STRING) {
+            s = v->string;
+        } else {
+            /* T_NODESET */
+            if (v->nodeset->used != 1) {
+                STATE_ERROR(state, PATHX_EMMATCH);
+                return;
+            }
+            s = v->nodeset->nodes[0]->value;
+        }
+        if (s != NULL) {
+            int r;
+            r = xstrtoint64(s, 10, &i);
+            if (r < 0) {
+                STATE_ERROR(state, PATHX_ENUMBER);
+                return;
+            }
+        }
+    }
+    state->value_pool[vind].number = i;
+    push_value(vind, state);
+}
+
 static struct regexp *
 nodeset_as_regexp(struct info *info, struct nodeset *ns, int glob) {
     struct regexp *result = NULL;
@@ -654,7 +700,9 @@ nodeset_as_regexp(struct info *info, struct nodeset *ns, int glob) {
     }
 
     if (used == 0) {
-        result = regexp_make_empty(info);
+        /* If the nodeset is empty, make sure we produce a regexp
+         * that never matches anything */
+        result = make_regexp_unescape(info, "[^\001-\7ff]", 0);
     } else {
         if (ALLOC_N(rx, ns->used) < 0)
             goto error;
@@ -1114,12 +1162,20 @@ static void ns_from_locpath(struct locpath *lp, uint *maxns,
         if (HAS_ERROR(state))
             goto error;
     }
+
     if (root == NULL) {
-        ns_add((*ns)[0], state->ctx, state);
+        struct step *first_step = NULL;
+        if (lp != NULL)
+            first_step = lp->steps;
+
+        struct tree *root_tree;
+        root_tree = step_root(first_step, state->ctx, state->root_ctx);
+        ns_add((*ns)[0], root_tree, state);
     } else {
         for (int i=0; i < root->used; i++)
             ns_add((*ns)[0], root->nodes[i], state);
     }
+
     if (HAS_ERROR(state))
         goto error;
 
@@ -2250,6 +2306,7 @@ int pathx_parse(const struct tree *tree,
                 const char *txt,
                 bool need_nodeset,
                 struct pathx_symtab *symtab,
+                struct tree *root_ctx,
                 struct pathx **pathx) {
     struct state *state = NULL;
 
@@ -2270,6 +2327,7 @@ int pathx_parse(const struct tree *tree,
     state->txt = txt;
     state->pos = txt;
     state->symtab = symtab;
+    state->root_ctx = root_ctx;
     state->error = err;
 
     if (ALLOC_N(state->value_pool, 8) < 0) {
@@ -2336,6 +2394,37 @@ static struct tree *tree_prev(struct tree *pos) {
     return node;
 }
 
+/* When the first step doesn't begin with ROOT then use relative root context
+ * instead. */
+static struct tree *step_root(struct step *step, struct tree *ctx,
+                              struct tree *root_ctx) {
+    struct tree *node = NULL;
+    switch (step->axis) {
+    case SELF:
+    case CHILD:
+    case DESCENDANT:
+    case PARENT:
+    case ANCESTOR:
+    case PRECEDING_SIBLING:
+    case FOLLOWING_SIBLING:
+        /* only use root_ctx when ctx is the absolute tree root */
+        if (ctx == ctx->parent && root_ctx != NULL)
+            node = root_ctx;
+        else
+            node = ctx;
+        break;
+    case ROOT:
+    case DESCENDANT_OR_SELF:
+        node = ctx;
+        break;
+    default:
+        assert(0);
+    }
+    if (node == NULL)
+        return NULL;
+    return node;
+}
+
 static struct tree *step_first(struct step *step, struct tree *ctx) {
     struct tree *node = NULL;
     switch (step->axis) {
