@@ -1,7 +1,7 @@
 /*
  * regexp.c:
  *
- * Copyright (C) 2007-2010 David Lutterkort
+ * Copyright (C) 2007-2011 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -56,7 +56,7 @@ char *regexp_escape(const struct regexp *r) {
 #endif
 
     if (pat == NULL)
-        pat = escape(r->pattern->str, -1);
+        pat = escape(r->pattern->str, -1, RX_ESCAPES);
 
     if (pat == NULL)
         return NULL;
@@ -115,7 +115,7 @@ void print_regexp(FILE *out, struct regexp *r) {
 
 struct regexp *
 make_regexp_unescape(struct info *info, const char *pat, int nocase) {
-    char *p = unescape(pat, strlen(pat));
+    char *p = unescape(pat, strlen(pat), NULL);
 
     if (p == NULL)
         return NULL;
