@@ -1,7 +1,7 @@
 /*
  * syntax.c:
  *
- * Copyright (C) 2007-2010 David Lutterkort
+ * Copyright (C) 2007-2011 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -64,7 +64,7 @@ const struct type *const t_unit      = &unit_type;
 
 static const char *const type_names[] = {
     "string", "regexp", "lens", "tree", "filter",
-    "transform", "unit", "function", NULL
+    "transform", "function", "unit", NULL
 };
 
 /* The anonymous identifier which we will never bind */
@@ -652,6 +652,10 @@ static void dump_ctx(struct ctx *ctx) {
  * Values
  */
 static void print_tree(FILE *out, int indent, struct tree *tree) {
+    if (tree == NULL) {
+        fprintf(out, "(null tree)\n");
+        return;
+    }
     list_for_each(t, tree) {
         for (int i=0; i < indent; i++) fputc(' ', out);
         fprintf(out, "{ ");
@@ -1447,11 +1451,10 @@ static struct value *compile_minus(struct term *exp, struct ctx *ctx) {
 }
 
 static struct value *compile_compose(struct term *exp, struct ctx *ctx) {
-    struct type *t = exp->type;
     struct info *info = exp->info;
     struct value *v;
 
-    if (t->tag == T_ARROW) {
+    if (exp->left->type->tag == T_ARROW) {
         // FIXME: This is really crufty, and should be desugared in the
         // parser so that we don't have to do all this manual type
         // computation. Should we write function compostion as
@@ -1472,7 +1475,16 @@ static struct value *compile_compose(struct term *exp, struct ctx *ctx) {
 
         struct term *func = build_func(param, app);
 
-        assert(type_equal(func->type, exp->type));
+        if (!type_equal(func->type, exp->type)) {
+            char *f = type_string(func->type);
+            char *e = type_string(exp->type);
+            fatal_error(info,
+              "Composition has type %s but should have type %s", f, e);
+            free(f);
+            free(e);
+            unref(func, term);
+            return exn_error();
+        }
         v = make_closure(func, ctx->local);
         unref(func, term);
     } else {
