@@ -1,7 +1,7 @@
 /*
  * transform.c: support for building and running transformers
  *
- * Copyright (C) 2007-2010 David Lutterkort
+ * Copyright (C) 2007-2011 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -527,7 +527,7 @@ static int load_file(struct augeas *aug, struct lens *lens,
     tree_replace(aug, path, tree);
 
     /* top level node span entire file length */
-    if (span != NULL) {
+    if (span != NULL && tree != NULL) {
         tree->parent->span = span;
         tree->parent->span->span_start = 0;
         tree->parent->span->span_end = text_len;
@@ -614,8 +614,17 @@ int transform_validate(struct augeas *aug, struct tree *xfm) {
     struct tree *l = NULL;
 
     for (struct tree *t = xfm->children; t != NULL; ) {
-        if (streqv(t->label, "lens"))
+        if (streqv(t->label, "lens")) {
             l = t;
+        } else if ((is_incl(t) || is_excl(t)) && t->value[0] != SEP) {
+            /* Normalize relative paths to absolute ones */
+            int r;
+            r = REALLOC_N(t->value, strlen(t->value) + 2);
+            ERR_NOMEM(r < 0, aug);
+            memmove(t->value + 1, t->value, strlen(t->value) + 1);
+            t->value[0] = SEP;
+        }
+
         if (streqv(t->label, "error")) {
             struct tree *del = t;
             t = del->next;
@@ -634,11 +643,12 @@ int transform_validate(struct augeas *aug, struct tree *xfm) {
         return -1;
     }
     lens_from_name(aug, l->value);
-    if (HAS_ERR(aug)) {
-        xfm_error(xfm, aug->error->details);
-        return -1;
-    }
+    ERR_BAIL(aug);
+
     return 0;
+ error:
+    xfm_error(xfm, aug->error->details);
+    return -1;
 }
 
 void transform_file_error(struct augeas *aug, const char *status,
@@ -852,8 +862,8 @@ static int file_saved_event(struct augeas *aug, const char *path) {
     struct tree *dummy;
     int r;
 
-    px = pathx_aug_parse(aug, aug->origin, AUGEAS_EVENTS_SAVED "[last()]",
-                         true);
+    px = pathx_aug_parse(aug, aug->origin, NULL,
+                         AUGEAS_EVENTS_SAVED "[last()]", true);
     ERR_BAIL(aug);
 
     if (pathx_find_one(px, &dummy) == 1) {
@@ -991,6 +1001,7 @@ int transform_save(struct augeas *aug, struct tree *xfm,
 
     if (fclose(fp) != 0) {
         err_status = "close_augnew";
+        fp = NULL;
         goto done;
     }
 
